from telegram import User


class UserURL:
    """Simle class to generate URL to user which can be sent in message.

    Usage
    -----
        ...
        # example
        user = update.message["reply_to_message"].from_user
        update.message.reply_text(
            f'User {UserURL(user)} has been muted.', 
            parse_mode="markdown"
        )
        ...
    """

    def __init__(self, user: User) -> None:
        self.user = user

    def __repr__(self) -> str:
        return f"[{self.user.first_name}](tg://user?id={self.user.id})"



from telegram import Update
from telegram.ext import CallbackContext
from . import bot


@bot.command("hello", help='reply "Hello, World!" text')
def hello(update: Update, context: CallbackContext):
    update.message.reply_text("Hello, World!")



from telegram.ext import CallbackContext
from telegram import Update
from uuid import uuid4


class InlineQuery:
    """Class for inline query.

    Usage
    -----
        @bot.inline_query
        def inline(query: telegrask.InlineQuery):
            # update, context = query.update, query.context
            ...
    """

    def __init__(self, update: Update, context: CallbackContext) -> None:
        self.update = update
        self.context = context
        self.query_id = self.update.inline_query.id
        self.query_str = str(self.update.inline_query.query)
        self.query = self.query_str.split(" ")
        self.answers = []

    def add_answer(self, inline_query_result) -> None:
        """Add new answer to answers list.

        Usage
        -----
            answer = telegram.InlineQueryResultArticle(...)  # or other InlineQueryResult* type
            query.add_answer(answer)
            ...
            # after all you need to send all added results to user
            query.send_answers()
        """

        self.answers.append(inline_query_result)

    def clear_answers(self) -> None:
        """Clear all answers.

        Usage
        -----
            query.clear_answers()
        """

        self.answers.clear()

    def send_answers(self) -> None:
        """Send answers to user.

        Usage
        -----
            # execute at the end of callback function
            query.send_answers()
        """

        self.context.bot.answer_inline_query(self.query_id, self.answers, cache_time=0)

    def simple_return(self, inline_query_result) -> None:
        """Add and send to user only one result.

        Usage
        -----
            answer = telegram.InlineQueryResultArticle(...)  # or other InlineQueryResult* type
            query.simple_return(answer)
        """

        self.add_answer(inline_query_result)
        self.send_answers()

    @staticmethod
    def parse_description(desc: str) -> str:
        if len(desc) > 30:
            desc = str(desc[:32] + (desc[29:] and "..."))
        return desc

    @staticmethod
    def get_random_id() -> str:
        return str(uuid4())



from telegram.ext import (
    Updater,
    CommandHandler,
    MessageHandler,
    InlineQueryHandler,
    CallbackContext,
    Filters,
)
from telegram import ParseMode, Update
from .exceptions import HelpPrasingError
from .helpparser import HelpParser
from .inlinequery import InlineQuery
from .config import Config
from typing import Union, Callable, Optional, Pattern


class Telegrask:
    """Main bot class.

    Usage
    -----
        bot = Telegrask(TOKEN)

        # optional configuration
        bot.config["ATTRIBUTE_NAME"] = attribute_value
    """

    default_config = Config({"HELP_MESSAGE": True})

    def __init__(self, token: str) -> None:
        self.config = self.default_config
        self.updater = Updater(token, use_context=True)
        self.dispatcher = self.updater.dispatcher
        self.help = HelpParser()

    def add_handler(self, handler) -> None:
        """Simple wrapper method to add custom handlers to dispatcher."""
        self.dispatcher.add_handler(handler)

    def command(
        self,
        commands: Union[str, list],
        help: Optional[str] = None,
        allow_without_prefix: bool = False,
    ) -> Callable:
        """Decorator for command callback function.

        Usage
        -----
            @bot.command("command_name", help="command help message")
            def callback_function(update, context):
                ...
        """

        def w(f):
            self.add_handler(CommandHandler(commands, f))
            command_name = commands[0] if type(commands) == list else commands
            if self.config["HELP_MESSAGE"]:
                if help is None:
                    raise HelpPrasingError("Help for command is not provided")
                self.help.add_command(command_name, help)

            if allow_without_prefix:
                self.message(Filters.text & ~Filters.command)(f)

        return w

    def message(self, filters: Filters) -> Callable:
        """Decorator for message callback function based on filters.

        Usage
        -----
            @bot.message(telegram.ext.filters.Filters.*)
            def callback_function(update, context):
                ...
        """

        return lambda f: self.add_handler(MessageHandler(filters, f))

    def message_regex(self, regex: Pattern) -> Callable:
        """Wrapper decorator for `self.message` used for handle messages
        by regexes.

        Usage
        -----
            @bot.message_regex("text or regex")
            def callback_function(update, context):
                ...
        """

        return lambda f: self.message(Filters.regex(regex))(f)

    def inline_query(self, f: Callable) -> None:
        """Decorator for inline query callback function.

        Usage
        -----
            @bot.inline_query
            def callback_function(query: telegrask.InlineQuery):
                # update, context = query.update, query.context
                ...
        """

        # Handle Update and CallbackContext and pass instance of InlineQuery class
        # as an argument for decorated function to avoid unnecessary code.
        self.add_handler(InlineQueryHandler(lambda u, c: f(InlineQuery(u, c))))

    def __help_command(self, update: Update, context: CallbackContext) -> None:
        """Send help message with description of each command."""

        update.message.reply_text(self.help.content, parse_mode=ParseMode.MARKDOWN)

    def custom_help_command(self, f: Callable) -> None:
        """Decorator to create custom help message instead generated by HelpParser.

        Usage
        -----
            @bot.custom_help_command
            def help_command(update, context, descriptions: dict):
                ...

            # @bot.custom_help_command is equivalent for:
            # @bot.command(["help", "start"], help="display this message")
            # with additional callback function parameter with commands descriptions
        """

        self.help.add_command("help", "display this message")
        self.add_handler(
            CommandHandler(
                ["help", "start"], lambda u, c: f(u, c, self.help.commands_descriptions)
            )
        )

    def run(self, debug: bool = False) -> None:
        """Start bot.

        Usage
        -----
            # execute at the end
            bot.run(debug=True)
        """

        if self.config["HELP_MESSAGE"]:
            self.command(["help", "start"], help="display this message")(
                self.__help_command
            )

        if debug:
            import logging

            logging.basicConfig(
                format="%(levelname)s - %(message)s", level=logging.DEBUG
            )
            logging.getLogger(__name__)

        self.updater.start_polling()
        self.updater.idle()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['telegrask', 'telegrask.cli', 'telegrask.cli.templates', 'telegrask.ext']

package_data = \
{'': ['*']}

install_requires = \
['python-telegram-bot>=13.7,<14.0']

setup_kwargs = {
    'name': 'telegrask',
    'version': '0.3.5',
    'description': 'Flask-inspired Telegram bot micro framework for Python.',
    'long_description': '# Telegrask\n\nFlask-inspired Telegram bot micro framework for Python. \nMain idea is to use callback function decorators and make bot \ncreating more intuitive for developer.\n\n## Installing\n\n```shell\n$ python3 -m pip install Telegrask\n```\n\n## Simple "Hello World" bot example\n\n```python\nfrom telegrask import Telegrask\n\nbot = Telegrask("BOT_TOKEN")\n\n\n@bot.command("hello", help=\'display "Hello, World!"\')\ndef hello_command(update, context):\n    update.message.reply_text("Hello, World!")\n\n\nif __name__ == "__main__":\n    bot.run(debug=True)\n```\n\nMore examples in [examples](./examples) folder.\n\n## Equivalent in pure [python-telegram-bot](https://python-telegram-bot.org/)\n\n```python\nfrom telegram.ext import Updater, CommandHandler\nfrom telegram import ParseMode\nimport logging\n\nlogging.basicConfig(format="%(levelname)s - %(message)s", level=logging.DEBUG)\nlogger = logging.getLogger(__name__)\n\n\ndef hello_command(update, context):\n    update.message.reply_text("Hello, World!")\n\n\ndef help_command(update, context):\n    help_content = """*Available commands*\n\n/hello\ndisplay "Hello, World!"\n\n/help\ndisplay this message\n"""\n    update.message.reply_text(help_content, parse_mode=ParseMode.MARKDOWN)\n\n\ndef main():\n    global updater\n    updater = Updater("BOT_TOKEN")\n    dispatcher = updater.dispatcher\n    dispatcher.add_handler(CommandHandler("hello", hello_command))\n    dispatcher.add_handler(CommandHandler(["help", "start"], help_command))\n    updater.start_polling()\n    updater.idle()\n\n\nif __name__ == "__main__":\n    main()\n```\n',
    'author': 'samedamci',
    'author_email': 'samedamci@disroot.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/samedamci/telegrask',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
